<?php
/**
 * Plugin Name:       Schema Manager
 * Plugin URI:        https://schema.zone/
 * Description:       Build awesome advanced schema and add directly to your website. Requires you have the Header Footer Code Manager installed and active. https://wordpress.org/plugins/header-footer-code-manager/
 * Version:           1.0
 * Requires at least: 5.2
 * Requires PHP:      7.2
 * Author URI:        https://schema.zone/
 * License:           GPL v2 or later
 */
 
//
// make sure HFCM is active
//
register_activation_hook( __FILE__, 'child_plugin_activate' );
function child_plugin_activate(){

    // Require parent plugin
    if ( ! is_plugin_active('header-footer-code-manager/99robots-header-footer-code-manager.php' ) and current_user_can( 'activate_plugins' ) ) {
        // Stop activation redirect and show error
        wp_die('Sorry, but this plugin requires the Header Footer Code Manager Plugin to be installed and active. <br><a href="' . admin_url( 'plugins.php' ) . '"> Return to Plugins</a>');
    }
}
 
 function hfcm_tasks($request){
	global $wpdb;
	// get the database name prefix used for this WordPress installation. We need that to access the HFCM table later on.
	$prefix = $wpdb->base_prefix;
	
	// are we doing a create/update or reading of the data?
	switch ( $request->get_method() ) {
		
		case 'POST':
			// creating or updating data in the database table
			$action = $request->get_param('action');
			
			// this is the payload which contains:
			// 1. the schema snippet
			// 2. the post type for which we're displaying the snippet (page or post) Note that a custom post type uses the post type of "post", and page uses "page"
			// 3. where the schema snippet gets injected (always the header)
			// 4. the post id to where this snippet will be used. Single page/post only, not site wide
			// 5. lastly, make the snippet active
			$info = $request->get_param('info');
			
			// when updating the table, we'll need the unique index of the entry
			$script_id = $request->get_param('script_id');
			
			if ($action == "create"){
				// write a new entry to table hfcm_scrips
				$res = $wpdb->insert($prefix . "hfcm_scripts", $info);	
			}else{
				// update existing entry in table hfcm_scripts
				$res = $wpdb->update($prefix . "hfcm_scripts", $info, $script_id);					
			}

			return $res;
			break;
			
		case 'GET':
			$res = $wpdb->get_results( "SELECT * FROM " . $prefix . "hfcm_scripts");
			return $res;
			break;
	}
}

//
// EXPLANATION
// 
// This WP hook allows us to use the previously declared function
// The POST method is declared for use so we can create or update schema in the HFCM database table
// the GET method is declared for use to we can pull out data from the HFCM database table
// 
// Both POST and GET methods are only available to a registered user who at least has editor capabilities.
//
// The REST route defined here is yourdomain.com/wp-json/erm/v1/hfcm
// the erm and hfcm parts are entirely custom. hfcm to internally identify this specific endpoint
// v1 as this is the first version of this route (no plans to make a v2!)
// 
// Read about the WordPress REST API here: https://developer.wordpress.org/rest-api/
// It is fully supported and uses application passwords in the WP admin -> users section
// 

add_action('rest_api_init', function() {
	register_rest_route('erm/v1', 'hfcm',
		array (
			array('methods'=> 'POST',
				 'callback' => 'hfcm_tasks',
				  'permission_callback' => function() {
					  return current_user_can('edit_others_posts');
				  }			 
			),
			array('methods'=> 'GET',
				 'callback' => 'hfcm_tasks',
				  'permission_callback' => function() {
					  return current_user_can('edit_others_posts');
				  }			 
			)
		)
	);
});

// This second piece comes in useful for adding the image information to the schema.
// If you have a featured image assigned to your page, this code adds the URI of that image to the WP REST get posts response.
// Now you saved a step of having to find an image link to add to the schema.

add_action('rest_api_init', 'register_rest_images' );

function register_rest_images(){
    register_rest_field( array('services'),
        'fimg_url',
        array(
            'get_callback'    => 'get_rest_featured_image',
            'update_callback' => null,
            'schema'          => null,
        )
    );
}
function get_rest_featured_image( $object, $field_name, $request ) {
    if( $object['featured_media'] ){
        $img = wp_get_attachment_image_src( $object['featured_media'], 'app-thumb' );
        return $img[0];
    }
    return false;
}


?>